# ---------------------------------------------------------------------------
# Dose_Response_Calculator.py
#
# Written by:
# STEVEN E. HANSER, U.S. Geological Survey, Forest and Rangeland Ecosystem Science Center, Boise, ID 83706, USA.
# Email: shanser@usgs.gov
# Phone: 208-426-2892
#
#CAMERON L. ALDRIDGE, NREL, Department of Ecosystem Science and Sustainability, Colorado State University, in cooperation with U.S. Geological Survey, Fort Collins Science Center, Fort Collins, CO 80526, USA.
#MATTHIAS LEU, Department of Biology, The College of William and Mary, Williamsburg, VA 23185.
#SCOTT E. NIELSEN, Department of Renewable Resources, University of Alberta, Edmonton, AB T6G 2H1, Canada.
#
#Although this program has been used by the USGS, no warranty, expressed or implied, is made by the USGS or the United States
#Government as to the accuracy and functioning of the program and related program material nor shall the fact of distribution
#constitute any such warranty, and no responsibility is assumed by the USGS in connection therewith.
#
# Usage: Dose_Response_Calculator.py <Predicted Response Raster> <Dose Explanatory Variable Raster> <Output Workspace> <Output Table> <Multiplier> <Graph Output Y/N>
#
# ---------------------------------------------------------------------------
import arcpy
import os
import sys
from arcpy.sa import *
# Check out any necessary licenses
arcpy.CheckOutExtension("spatial")

ResponseRaster = arcpy.GetParameterAsText(0)
DoseVariable = arcpy.GetParameterAsText(1)
Output_Workspace = arcpy.GetParameterAsText(2)
Output_Table_Name = arcpy.GetParameterAsText(3)
Multiplier = arcpy.GetParameter(4)
GraphOutput = arcpy.GetParameter(5)
DVFolder = Output_Workspace

scriptPath = sys.path[0]
GraphTemplatePath = scriptPath + os.path.sep + "Dose Response Curve.grf"
OutputTable = Output_Workspace + os.path.sep + Output_Table_Name + ".dbf"

desc = arcpy.Describe(DoseVariable)
FullName = desc.catalogPath
desc = arcpy.Describe(ResponseRaster)
InputFullName = desc.catalogPath
DVName = (os.path.split(FullName)[1])
InputName = (os.path.split(InputFullName)[1])
DoseVariable_INT = DVFolder + os.path.sep + "IntegerDoseVariable" + os.path.sep + DVName
Output_arc_graph_name = Output_Workspace + os.path.sep + Output_Table_Name + "_DoseResponseCurve.grf"
Output_graph_file = Output_Workspace + os.path.sep + Output_Table_Name + "_DoseResponseCurve.jpg"
  
if arcpy.Exists(DoseVariable_INT):
    arcpy.AddMessage ("Running Zonal Statatistics")
    arcpy.AddMessage ("    Input: " + InputName)
    arcpy.AddMessage ("    Zones: " + DVName)
    arcpy.gp.ZonalStatisticsAsTable(DoseVariable_INT, "Value", ResponseRaster, OutputTable, "DATA")
    arcpy.AddMessage ("    Adding SD_L = Mean - 1 SD")
    arcpy.AddField_management(OutputTable, "SD_L", "DOUBLE")
    arcpy.CalculateField_management(OutputTable, "SD_L", "!MEAN! - !STD!", "PYTHON_9.3", "")
    arcpy.AddMessage ("    Adding SD_U = Mean + 1 SD")
    arcpy.AddField_management(OutputTable, "SD_U", "DOUBLE")
    arcpy.CalculateField_management(OutputTable, "SD_U", "!MEAN! + !STD!", "PYTHON_9.3", "")

else:
    if arcpy.Exists(DVFolder + os.path.sep + "IntegerDoseVariable"):
        arcpy.AddMessage ("Creating response variable: " + DVName)
        r = Int(Raster(DoseVariable) * Multiplier)
        r.save(DoseVariable_INT)
        arcpy.AddMessage ("Running Zonal Statatistics")
        arcpy.AddMessage ("    Input: " + InputName)
        arcpy.AddMessage ("    Zones: " + DVName)
        arcpy.gp.ZonalStatisticsAsTable(DoseVariable_INT, "Value", ResponseRaster, OutputTable, "DATA")
        arcpy.AddMessage ("    Adding SD_L = Mean - 1 SD")
        arcpy.AddField_management(OutputTable, "SD_L", "DOUBLE")
        arcpy.CalculateField_management(OutputTable, "SD_L", "!MEAN! - !STD!", "PYTHON_9.3", "")
        arcpy.AddMessage ("    Adding SD_U = Mean + 1 SD")
        arcpy.AddField_management(OutputTable, "SD_U", "DOUBLE")
        arcpy.CalculateField_management(OutputTable, "SD_U", "!MEAN! + !STD!", "PYTHON_9.3", "")
    else:
        arcpy.CreateFolder_management(DVFolder, "IntegerDoseVariable")
        arcpy.AddMessage ("Creating response variable: " + DVName)
        r = Int(Raster(DoseVariable) * Multiplier)
        r.save(DoseVariable_INT)
        arcpy.AddMessage ("Running Zonal Statatistics")
        arcpy.AddMessage ("    Input: " + InputName)
        arcpy.AddMessage ("    Zones: " + DVName)
        arcpy.gp.ZonalStatisticsAsTable(DoseVariable_INT, "Value", ResponseRaster, OutputTable, "DATA")
        arcpy.AddMessage ("    Adding SD_L = Mean - 1 SD")
        arcpy.AddField_management(OutputTable, "SD_L", "DOUBLE")
        arcpy.CalculateField_management(OutputTable, "SD_L", "!MEAN! - !STD!", "PYTHON_9.3", "")
        arcpy.AddMessage ("    Adding SD_U = Mean + 1 SD")
        arcpy.AddField_management(OutputTable, "SD_U", "DOUBLE")
        arcpy.CalculateField_management(OutputTable, "SD_U", "!MEAN! + !STD!", "PYTHON_9.3", "")

if GraphOutput == 1:
    arcpy.AddMessage ("Creating graph output: " + Output_graph_file)    
    arcpy.MakeGraph_management(GraphTemplatePath, "SERIES=line:vertical DATA=" + OutputTable + " X=VALUE Y=MEAN;SERIES=line:vertical DATA=" + OutputTable + " X=VALUE Y=SD_L;SERIES=line:vertical DATA=" + OutputTable + " X=VALUE Y=SD_U;GRAPH=general TITLE=Dose Response Curve;LEGEND=general;AXIS=left TITLE=" + InputName + ";AXIS=right;AXIS=bottom TITLE=" + DVName + ";AXIS=top", Output_arc_graph_name)
    arcpy.SaveGraph_management(Output_arc_graph_name, Output_graph_file, "MAINTAIN_ASPECT_RATIO", 1000)

